package java.time.chrono;

import java.time.Instant;
import java.time.ZoneOffset;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoField;
import java.time.temporal.TemporalQuery;
import java.time.temporal.TemporalAmount;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalUnit;
import java.time.temporal.TemporalField;
import java.time.LocalTime;
import java.time.DateTimeException;
import java.time.temporal.TemporalQueries;
import java.util.Objects;
import java.time.temporal.TemporalAccessor;
import java.util.Comparator;
import java.time.temporal.TemporalAdjuster;
import java.time.temporal.Temporal;

public interface ChronoLocalDateTime<D extends ChronoLocalDate> extends Temporal, TemporalAdjuster, Comparable<ChronoLocalDateTime<?>>
{
    public static Comparator<ChronoLocalDateTime<?>> timeLineOrder() {
        return AbstractChronology.DATE_TIME_ORDER;
    }
    
    public static ChronoLocalDateTime<?> from(TemporalAccessor obj) {
        if ((obj instanceof ChronoLocalDateTime)) {
            return (ChronoLocalDateTime)obj;
        }
        Objects.requireNonNull(obj, "temporal");
        Chronology chronology = (Chronology)obj.query(TemporalQueries.chronology());
        if (chronology == null) {
            throw new DateTimeException("Unable to obtain ChronoLocalDateTime from TemporalAccessor: " + obj.getClass());
        }
        return chronology.localDateTime(obj);
    }
    
    default public Chronology getChronology() {
        return this.toLocalDate().getChronology();
    }
    
    public abstract D toLocalDate();
    
    public abstract LocalTime toLocalTime();
    
    public abstract boolean isSupported(TemporalField p0);
    
    default public boolean isSupported(TemporalUnit temporalUnit) {
        if ((temporalUnit instanceof ChronoUnit)) {
            return temporalUnit != ChronoUnit.FOREVER;
        }
        return temporalUnit != null && temporalUnit.isSupportedBy(this);
    }
    
    default public ChronoLocalDateTime<D> with(TemporalAdjuster adjuster) {
        return ChronoLocalDateTimeImpl.ensureValid(this.getChronology(), super.with(adjuster));
    }
    
    public abstract ChronoLocalDateTime<D> with(TemporalField p0, long p1);
    
    default public ChronoLocalDateTime<D> plus(TemporalAmount amount) {
        return ChronoLocalDateTimeImpl.ensureValid(this.getChronology(), super.plus(amount));
    }
    
    public abstract ChronoLocalDateTime<D> plus(long p0, TemporalUnit p1);
    
    default public ChronoLocalDateTime<D> minus(TemporalAmount amount) {
        return ChronoLocalDateTimeImpl.ensureValid(this.getChronology(), super.minus(amount));
    }
    
    default public ChronoLocalDateTime<D> minus(long amountToSubtract, TemporalUnit unit) {
        return ChronoLocalDateTimeImpl.ensureValid(this.getChronology(), super.minus(amountToSubtract, unit));
    }
    
    default public <R> R query(TemporalQuery<R> temporalQuery) {
        if (temporalQuery == TemporalQueries.zoneId() || temporalQuery == TemporalQueries.zone() || temporalQuery == TemporalQueries.offset()) {
            return null;
        }
        if (temporalQuery == TemporalQueries.localTime()) {
            return this.toLocalTime();
        }
        if (temporalQuery == TemporalQueries.chronology()) {
            return this.getChronology();
        }
        if (temporalQuery == TemporalQueries.precision()) {
            return ChronoUnit.NANOS;
        }
        return temporalQuery.queryFrom(this);
    }
    
    default public Temporal adjustInto(Temporal temporal) {
        return temporal.with(ChronoField.EPOCH_DAY, this.toLocalDate().toEpochDay()).with(ChronoField.NANO_OF_DAY, this.toLocalTime().toNanoOfDay());
    }
    
    default public String format(DateTimeFormatter obj) {
        Objects.requireNonNull(obj, "formatter");
        return obj.format(this);
    }
    
    public abstract ChronoZonedDateTime<D> atZone(ZoneId p0);
    
    default public Instant toInstant(ZoneOffset zoneOffset) {
        return Instant.ofEpochSecond(this.toEpochSecond(zoneOffset), (long)this.toLocalTime().getNano());
    }
    
    default public long toEpochSecond(ZoneOffset obj) {
        Objects.requireNonNull(obj, "offset");
        return this.toLocalDate().toEpochDay() * 86400L + (long)this.toLocalTime().toSecondOfDay() - (long)obj.getTotalSeconds();
    }
    
    default public int compareTo(ChronoLocalDateTime<?> chronoLocalDateTime) {
        int n = this.toLocalDate().compareTo(chronoLocalDateTime.toLocalDate());
        if (n == 0) {
            n = this.toLocalTime().compareTo(chronoLocalDateTime.toLocalTime());
            if (n == 0) {
                n = this.getChronology().compareTo(chronoLocalDateTime.getChronology());
            }
        }
        return n;
    }
    
    default public boolean isAfter(ChronoLocalDateTime<?> chronoLocalDateTime) {
        long epochDay = this.toLocalDate().toEpochDay();
        long epochDay2 = chronoLocalDateTime.toLocalDate().toEpochDay();
        return epochDay > epochDay2 || (epochDay == epochDay2 && this.toLocalTime().toNanoOfDay() > chronoLocalDateTime.toLocalTime().toNanoOfDay());
    }
    
    default public boolean isBefore(ChronoLocalDateTime<?> chronoLocalDateTime) {
        long epochDay = this.toLocalDate().toEpochDay();
        long epochDay2 = chronoLocalDateTime.toLocalDate().toEpochDay();
        return epochDay < epochDay2 || (epochDay == epochDay2 && this.toLocalTime().toNanoOfDay() < chronoLocalDateTime.toLocalTime().toNanoOfDay());
    }
    
    default public boolean isEqual(ChronoLocalDateTime<?> chronoLocalDateTime) {
        return this.toLocalTime().toNanoOfDay() == chronoLocalDateTime.toLocalTime().toNanoOfDay() && this.toLocalDate().toEpochDay() == chronoLocalDateTime.toLocalDate().toEpochDay();
    }
    
    public abstract boolean equals(Object p0);
    
    public abstract int hashCode();
    
    public abstract String toString();
}
